@@ -1,20 +1,24 @@
-# GPL # "author": "Buerbaum Martin (Pontiac)"
+# SPDX-License-Identifier: GPL-2.0-or-later
 
-import bpy
+# Author: Buerbaum Martin (Pontiac)
+
+import bpy, bmesh
 from math import sin, cos, tan, pi, radians
 from bpy.types import Operator
 from bpy.props import (
         FloatProperty,
         IntProperty,
+        BoolProperty,
+        StringProperty,
         )
-
+from bpy_extras import object_utils
 
 # Create a new mesh (object) from verts/edges/faces.
 # verts/edges/faces ... List of vertices/edges/faces for the
 #                       new mesh (as used in from_pydata)
 # name ... Name of the new mesh (& object)
 
-def create_mesh_object(context, verts, edges, faces, name):
+def create_mesh(context, verts, edges, faces, name):
     # Create new mesh
     mesh = bpy.data.meshes.new(name)
 
@@ -24,9 +28,7 @@ def create_mesh_object(context, verts, edges, faces, name):
     # Update mesh geometry after adding stuff.
     mesh.update()
 
-    from bpy_extras import object_utils
-    return object_utils.object_data_add(context, mesh, operator=None)
-
+    return mesh
 
 # A very simple "bridge" tool.
 
@@ -87,13 +89,31 @@ def createFaces(vertIdx1, vertIdx2, closed=False, flipped=False):
 
 
 # Create the vertices and polygons for a simple elbow (bent pipe)
+def ElbowJointParameters():
+    ElbowJointParameters = [
+    "radius",
+    "div",
+    "angle",
+    "startLength",
+    "endLength",
+    ]
+    return ElbowJointParameters
 
-class AddElbowJoint(Operator):
+class AddElbowJoint(Operator, object_utils.AddObjectHelper):
     bl_idname = "mesh.primitive_elbow_joint_add"
     bl_label = "Add Pipe Elbow"
     bl_description = "Construct an elbow pipe mesh"
     bl_options = {'REGISTER', 'UNDO', 'PRESET'}
 
+    ElbowJoint : BoolProperty(name = "ElbowJoint",
+                default = True,
+                description = "ElbowJoint")
+
+    #### change properties
+    change : BoolProperty(name = "Change",
+                default = False,
+                description = "change ElbowJoint")
+
     radius: FloatProperty(
         name="Radius",
         description="The radius of the pipe",
@@ -133,7 +153,28 @@ class AddElbowJoint(Operator):
         unit="LENGTH"
         )
 
+    def draw(self, context):
+        layout = self.layout
+
+        box = layout.box()
+        box.prop(self, 'radius')
+        box.prop(self, 'div')
+        box.prop(self, 'angle')
+        box.prop(self, 'startLength')
+        box.prop(self, 'endLength')
+
+        if self.change == False:
+            # generic transform props
+            box = layout.box()
+            box.prop(self, 'align', expand=True)
+            box.prop(self, 'location', expand=True)
+            box.prop(self, 'rotation', expand=True)
+
     def execute(self, context):
+        # turn off 'Enter Edit Mode'
+        use_enter_edit_mode = bpy.context.preferences.edit.use_enter_edit_mode
+        bpy.context.preferences.edit.use_enter_edit_mode = False
+
         radius = self.radius
         div = self.div
 
@@ -189,20 +230,80 @@ class AddElbowJoint(Operator):
         faces.extend(createFaces(loop1, loop2, closed=True))
         faces.extend(createFaces(loop2, loop3, closed=True))
 
-        base = create_mesh_object(context, verts, [], faces, "Elbow Joint")
+        if bpy.context.mode == "OBJECT":
+            if (context.selected_objects != []) and context.active_object and \
+                (context.active_object.data is not None) and ('ElbowJoint' in context.active_object.data.keys()) and \
+                (self.change == True):
+                obj = context.active_object
+                oldmesh = obj.data
+                oldmeshname = obj.data.name
+                mesh = create_mesh(context, verts, [], faces, "Elbow Joint")
+                obj.data = mesh
+                for material in oldmesh.materials:
+                    obj.data.materials.append(material)
+                bpy.data.meshes.remove(oldmesh)
+                obj.data.name = oldmeshname
+            else:
+                mesh = create_mesh(context, verts, [], faces, "Elbow Joint")
+                obj = object_utils.object_data_add(context, mesh, operator=self)
+
+            mesh.update()
+
+            obj.data["ElbowJoint"] = True
+            obj.data["change"] = False
+            for prm in ElbowJointParameters():
+                obj.data[prm] = getattr(self, prm)
+
+        if bpy.context.mode == "EDIT_MESH":
+            active_object = context.active_object
+            name_active_object = active_object.name
+            bpy.ops.object.mode_set(mode='OBJECT')
+            mesh = create_mesh(context, verts, [], faces, "TMP")
+            obj = object_utils.object_data_add(context, mesh, operator=self)
+            obj.select_set(True)
+            active_object.select_set(True)
+            bpy.context.view_layer.objects.active = active_object
+            bpy.ops.object.join()
+            context.active_object.name = name_active_object
+            bpy.ops.object.mode_set(mode='EDIT')
+
+        if use_enter_edit_mode:
+            bpy.ops.object.mode_set(mode = 'EDIT')
+
+        # restore pre operator state
+        bpy.context.preferences.edit.use_enter_edit_mode = use_enter_edit_mode
 
         return {'FINISHED'}
 
 
 # Create the vertices and polygons for a simple tee (T) joint
 # The base arm of the T can be positioned in an angle if needed though
+def TeeJointParameters():
+    TeeJointParameters = [
+    "radius",
+    "div",
+    "angle",
+    "startLength",
+    "endLength",
+    "branchLength",
+    ]
+    return TeeJointParameters
 
-class AddTeeJoint(Operator):
+class AddTeeJoint(Operator, object_utils.AddObjectHelper):
     bl_idname = "mesh.primitive_tee_joint_add"
     bl_label = "Add Pipe Tee-Joint"
     bl_description = "Construct a tee-joint pipe mesh"
     bl_options = {'REGISTER', 'UNDO', 'PRESET'}
 
+    TeeJoint : BoolProperty(name = "TeeJoint",
+                default = True,
+                description = "TeeJoint")
+
+    #### change properties
+    change : BoolProperty(name = "Change",
+                default = False,
+                description = "change TeeJoint")
+
     radius: FloatProperty(
         name="Radius",
         description="The radius of the pipe",
@@ -254,7 +355,29 @@ class AddTeeJoint(Operator):
         unit="LENGTH"
         )
 
+    def draw(self, context):
+        layout = self.layout
+
+        box = layout.box()
+        box.prop(self, 'radius')
+        box.prop(self, 'div')
+        box.prop(self, 'angle')
+        box.prop(self, 'startLength')
+        box.prop(self, 'endLength')
+        box.prop(self, 'branchLength')
+
+        if self.change == False:
+            # generic transform props
+            box = layout.box()
+            box.prop(self, 'align', expand=True)
+            box.prop(self, 'location', expand=True)
+            box.prop(self, 'rotation', expand=True)
+
     def execute(self, context):
+        # turn off 'Enter Edit Mode'
+        use_enter_edit_mode = bpy.context.preferences.edit.use_enter_edit_mode
+        bpy.context.preferences.edit.use_enter_edit_mode = False
+
         radius = self.radius
         div = self.div
 
@@ -374,17 +497,78 @@ class AddTeeJoint(Operator):
         faces.extend(createFaces(loopJoint2, loopArm, closed=True))
         faces.extend(createFaces(loopJoint3, loopMainEnd, closed=True))
 
-        base = create_mesh_object(context, verts, [], faces, "Tee Joint")
+        if bpy.context.mode == "OBJECT":
+            if (context.selected_objects != []) and context.active_object and \
+                (context.active_object.data is not None) and ('TeeJoint' in context.active_object.data.keys()) and \
+                (self.change == True):
+                obj = context.active_object
+                oldmesh = obj.data
+                oldmeshname = obj.data.name
+                mesh = create_mesh(context, verts, [], faces, "Tee Joint")
+                obj.data = mesh
+                for material in oldmesh.materials:
+                    obj.data.materials.append(material)
+                bpy.data.meshes.remove(oldmesh)
+                obj.data.name = oldmeshname
+            else:
+                mesh = create_mesh(context, verts, [], faces, "Tee Joint")
+                obj = object_utils.object_data_add(context, mesh, operator=self)
+
+            mesh.update()
+
+            obj.data["TeeJoint"] = True
+            obj.data["change"] = False
+            for prm in TeeJointParameters():
+                obj.data[prm] = getattr(self, prm)
+
+        if bpy.context.mode == "EDIT_MESH":
+            active_object = context.active_object
+            name_active_object = active_object.name
+            bpy.ops.object.mode_set(mode='OBJECT')
+            mesh = create_mesh(context, verts, [], faces, "TMP")
+            obj = object_utils.object_data_add(context, mesh, operator=self)
+            obj.select_set(True)
+            active_object.select_set(True)
+            bpy.context.view_layer.objects.active = active_object
+            bpy.ops.object.join()
+            context.active_object.name = name_active_object
+            bpy.ops.object.mode_set(mode='EDIT')
+
+        if use_enter_edit_mode:
+            bpy.ops.object.mode_set(mode = 'EDIT')
+
+        # restore pre operator state
+        bpy.context.preferences.edit.use_enter_edit_mode = use_enter_edit_mode
 
         return {'FINISHED'}
 
+def WyeJointParameters():
+    WyeJointParameters = [
+    "radius",
+    "div",
+    "angle1",
+    "angle2",
+    "startLength",
+    "branch1Length",
+    "branch2Length",
+    ]
+    return WyeJointParameters
 
-class AddWyeJoint(Operator):
+class AddWyeJoint(Operator, object_utils.AddObjectHelper):
     bl_idname = "mesh.primitive_wye_joint_add"
     bl_label = "Add Pipe Wye-Joint"
     bl_description = "Construct a wye-joint pipe mesh"
     bl_options = {'REGISTER', 'UNDO', 'PRESET'}
 
+    WyeJoint : BoolProperty(name = "WyeJoint",
+                default = True,
+                description = "WyeJoint")
+
+    #### change properties
+    change : BoolProperty(name = "Change",
+                default = False,
+                description = "change WyeJoint")
+
     radius: FloatProperty(
         name="Radius",
         description="The radius of the pipe",
@@ -444,7 +628,30 @@ class AddWyeJoint(Operator):
         unit="LENGTH"
         )
 
+    def draw(self, context):
+        layout = self.layout
+
+        box = layout.box()
+        box.prop(self, 'radius')
+        box.prop(self, 'div')
+        box.prop(self, 'angle1')
+        box.prop(self, 'angle2')
+        box.prop(self, 'startLength')
+        box.prop(self, 'branch1Length')
+        box.prop(self, 'branch2Length')
+
+        if self.change == False:
+            # generic transform props
+            box = layout.box()
+            box.prop(self, 'align', expand=True)
+            box.prop(self, 'location', expand=True)
+            box.prop(self, 'rotation', expand=True)
+
     def execute(self, context):
+        # turn off 'Enter Edit Mode'
+        use_enter_edit_mode = bpy.context.preferences.edit.use_enter_edit_mode
+        bpy.context.preferences.edit.use_enter_edit_mode = False
+
         radius = self.radius
         div = self.div
 
@@ -574,19 +781,82 @@ class AddWyeJoint(Operator):
         faces.extend(createFaces(loopJoint2, loopArm1, closed=True))
         faces.extend(createFaces(loopJoint3, loopArm2, closed=True))
 
-        base = create_mesh_object(context, verts, [], faces, "Wye Joint")
+        if bpy.context.mode == "OBJECT":
+            if (context.selected_objects != []) and context.active_object and \
+                (context.active_object.data is not None) and ('WyeJoint' in context.active_object.data.keys()) and \
+                (self.change == True):
+                obj = context.active_object
+                oldmesh = obj.data
+                oldmeshname = obj.data.name
+                mesh = create_mesh(context, verts, [], faces, "Wye Joint")
+                obj.data = mesh
+                for material in oldmesh.materials:
+                    obj.data.materials.append(material)
+                bpy.data.meshes.remove(oldmesh)
+                obj.data.name = oldmeshname
+            else:
+                mesh = create_mesh(context, verts, [], faces, "Wye Joint")
+                obj = object_utils.object_data_add(context, mesh, operator=self)
+
+            mesh.update()
+
+            obj.data["WyeJoint"] = True
+            obj.data["change"] = False
+            for prm in WyeJointParameters():
+                obj.data[prm] = getattr(self, prm)
+
+        if bpy.context.mode == "EDIT_MESH":
+            active_object = context.active_object
+            name_active_object = active_object.name
+            bpy.ops.object.mode_set(mode='OBJECT')
+            mesh = create_mesh(context, verts, [], faces, "TMP")
+            obj = object_utils.object_data_add(context, mesh, operator=self)
+            obj.select_set(True)
+            active_object.select_set(True)
+            bpy.context.view_layer.objects.active = active_object
+            bpy.ops.object.join()
+            context.active_object.name = name_active_object
+            bpy.ops.object.mode_set(mode='EDIT')
+
+        if use_enter_edit_mode:
+            bpy.ops.object.mode_set(mode = 'EDIT')
+
+        # restore pre operator state
+        bpy.context.preferences.edit.use_enter_edit_mode = use_enter_edit_mode
 
         return {'FINISHED'}
 
 
 # Create the vertices and polygons for a cross (+ or X) pipe joint
+def CrossJointParameters():
+    CrossJointParameters = [
+    "radius",
+    "div",
+    "angle1",
+    "angle2",
+    "angle3",
+    "startLength",
+    "branch1Length",
+    "branch2Length",
+    "branch3Length",
+    ]
+    return CrossJointParameters
 
-class AddCrossJoint(Operator):
+class AddCrossJoint(Operator, object_utils.AddObjectHelper):
     bl_idname = "mesh.primitive_cross_joint_add"
     bl_label = "Add Pipe Cross-Joint"
     bl_description = "Construct a cross-joint pipe mesh"
     bl_options = {'REGISTER', 'UNDO', 'PRESET'}
 
+    CrossJoint : BoolProperty(name = "CrossJoint",
+                default = True,
+                description = "CrossJoint")
+
+    #### change properties
+    change : BoolProperty(name = "Change",
+                default = False,
+                description = "change CrossJoint")
+
     radius: FloatProperty(
         name="Radius",
         description="The radius of the pipe",
@@ -657,7 +927,32 @@ class AddCrossJoint(Operator):
         unit="LENGTH"
         )
 
+    def draw(self, context):
+        layout = self.layout
+
+        box = layout.box()
+        box.prop(self, 'radius')
+        box.prop(self, 'div')
+        box.prop(self, 'angle1')
+        box.prop(self, 'angle2')
+        box.prop(self, 'angle3')
+        box.prop(self, 'startLength')
+        box.prop(self, 'branch1Length')
+        box.prop(self, 'branch2Length')
+        box.prop(self, 'branch3Length')
+
+        if self.change == False:
+            # generic transform props
+            box = layout.box()
+            box.prop(self, 'align', expand=True)
+            box.prop(self, 'location', expand=True)
+            box.prop(self, 'rotation', expand=True)
+
     def execute(self, context):
+        # turn off 'Enter Edit Mode'
+        use_enter_edit_mode = bpy.context.preferences.edit.use_enter_edit_mode
+        bpy.context.preferences.edit.use_enter_edit_mode = False
+
         radius = self.radius
         div = self.div
 
@@ -834,19 +1129,77 @@ class AddCrossJoint(Operator):
         faces.extend(createFaces(loopJoint3, loopArm2, closed=True))
         faces.extend(createFaces(loopJoint4, loopArm3, closed=True))
 
-        base = create_mesh_object(context, verts, [], faces, "Cross Joint")
+        if bpy.context.mode == "OBJECT":
+            if (context.selected_objects != []) and context.active_object and \
+                (context.active_object.data is not None) and ('CrossJoint' in context.active_object.data.keys()) and \
+                (self.change == True):
+                obj = context.active_object
+                oldmesh = obj.data
+                oldmeshname = obj.data.name
+                mesh = create_mesh(context, verts, [], faces, "Cross Joint")
+                obj.data = mesh
+                for material in oldmesh.materials:
+                    obj.data.materials.append(material)
+                bpy.data.meshes.remove(oldmesh)
+                obj.data.name = oldmeshname
+            else:
+                mesh = create_mesh(context, verts, [], faces, "Cross Joint")
+                obj = object_utils.object_data_add(context, mesh, operator=self)
+
+            mesh.update()
+
+            obj.data["CrossJoint"] = True
+            obj.data["change"] = False
+            for prm in CrossJointParameters():
+                obj.data[prm] = getattr(self, prm)
+
+        if bpy.context.mode == "EDIT_MESH":
+            active_object = context.active_object
+            name_active_object = active_object.name
+            bpy.ops.object.mode_set(mode='OBJECT')
+            mesh = create_mesh(context, verts, [], faces, "TMP")
+            obj = object_utils.object_data_add(context, mesh, operator=self)
+            obj.select_set(True)
+            active_object.select_set(True)
+            bpy.context.view_layer.objects.active = active_object
+            bpy.ops.object.join()
+            context.active_object.name = name_active_object
+            bpy.ops.object.mode_set(mode='EDIT')
+
+        if use_enter_edit_mode:
+            bpy.ops.object.mode_set(mode = 'EDIT')
+
+        # restore pre operator state
+        bpy.context.preferences.edit.use_enter_edit_mode = use_enter_edit_mode
 
         return {'FINISHED'}
 
 
 # Create the vertices and polygons for a regular n-joint
+def NJointParameters():
+    NJointParameters = [
+    "radius",
+    "div",
+    "number",
+    "length",
+    ]
+    return NJointParameters
 
-class AddNJoint(Operator):
+class AddNJoint(Operator, object_utils.AddObjectHelper):
     bl_idname = "mesh.primitive_n_joint_add"
     bl_label = "Add Pipe N-Joint"
     bl_description = "Construct a n-joint pipe mesh"
     bl_options = {'REGISTER', 'UNDO', 'PRESET'}
 
+    NJoint : BoolProperty(name = "NJoint",
+                default = True,
+                description = "NJoint")
+
+    #### change properties
+    change : BoolProperty(name = "Change",
+                default = False,
+                description = "change NJoint")
+
     radius: FloatProperty(
         name="Radius",
         description="The radius of the pipe",
@@ -878,7 +1231,27 @@ class AddNJoint(Operator):
         unit="LENGTH"
         )
 
+    def draw(self, context):
+        layout = self.layout
+
+        box = layout.box()
+        box.prop(self, 'radius')
+        box.prop(self, 'div')
+        box.prop(self, 'number')
+        box.prop(self, 'length')
+
+        if self.change == False:
+            # generic transform props
+            box = layout.box()
+            box.prop(self, 'align', expand=True)
+            box.prop(self, 'location', expand=True)
+            box.prop(self, 'rotation', expand=True)
+
     def execute(self, context):
+        # turn off 'Enter Edit Mode'
+        use_enter_edit_mode = bpy.context.preferences.edit.use_enter_edit_mode
+        bpy.context.preferences.edit.use_enter_edit_mode = False
+
         radius = self.radius
         div = self.div
         number = self.number
@@ -1001,6 +1374,45 @@ class AddNJoint(Operator):
                 createFaces(loopsJoints[loopIdx],
                 loopsEndCircles[loopIdx], closed=True))
 
-        base = create_mesh_object(context, verts, [], faces, "N Joint")
+        if bpy.context.mode == "OBJECT":
+            if (context.selected_objects != []) and context.active_object and \
+                (context.active_object.data is not None) and ('NJoint' in context.active_object.data.keys()) and \
+                (self.change == True):
+                obj = context.active_object
+                oldmesh = obj.data
+                oldmeshname = obj.data.name
+                mesh = create_mesh(context, verts, [], faces, "N Joint")
+                obj.data = mesh
+                for material in oldmesh.materials:
+                    obj.data.materials.append(material)
+                bpy.data.meshes.remove(oldmesh)
+                obj.data.name = oldmeshname
+            else:
+                mesh = create_mesh(context, verts, [], faces, "N Joint")
+                obj = object_utils.object_data_add(context, mesh, operator=self)
+
+            obj.data["NJoint"] = True
+            obj.data["change"] = False
+            for prm in NJointParameters():
+                obj.data[prm] = getattr(self, prm)
+
+        if bpy.context.mode == "EDIT_MESH":
+            active_object = context.active_object
+            name_active_object = active_object.name
+            bpy.ops.object.mode_set(mode='OBJECT')
+            mesh = create_mesh(context, verts, [], faces, "TMP")
+            obj = object_utils.object_data_add(context, mesh, operator=self)
+            obj.select_set(True)
+            active_object.select_set(True)
+            bpy.context.view_layer.objects.active = active_object
+            bpy.ops.object.join()
+            context.active_object.name = name_active_object
+            bpy.ops.object.mode_set(mode='EDIT')
+
+        if use_enter_edit_mode:
+            bpy.ops.object.mode_set(mode = 'EDIT')
+
+        # restore pre operator state
+        bpy.context.preferences.edit.use_enter_edit_mode = use_enter_edit_mode
 
         return {'FINISHED'}
