@@ -29,7 +29,8 @@ ccl_device float light_tree_cos_bounding_box_angle(const BoundingBox bbox,
                                                    const float3 point_to_centroid)
 {
   if (P.x > bbox.min.x && P.y > bbox.min.y && P.z > bbox.min.z && P.x < bbox.max.x &&
-      P.y < bbox.max.y && P.z < bbox.max.z) {
+      P.y < bbox.max.y && P.z < bbox.max.z)
+  {
     /* If P is inside the bbox, `theta_u` covers the whole sphere. */
     return -1.0f;
   }
@@ -69,6 +70,59 @@ ccl_device float3 compute_v(
                                                     cos_phi0 * o0 + dot_o1_a * inv_len * o1;
 }
 
+ccl_device_inline bool is_light(const ccl_global KernelLightTreeEmitter *kemitter)
+{
+  return kemitter->light.id < 0;
+}
+
+ccl_device_inline bool is_mesh(const ccl_global KernelLightTreeEmitter *kemitter)
+{
+  return !is_light(kemitter) && kemitter->mesh_light.object_id == OBJECT_NONE;
+}
+
+ccl_device_inline bool is_triangle(const ccl_global KernelLightTreeEmitter *kemitter)
+{
+  return !is_light(kemitter) && kemitter->mesh_light.object_id != OBJECT_NONE;
+}
+
+ccl_device_inline bool is_leaf(const ccl_global KernelLightTreeNode *knode)
+{
+  /* The distant node is also considered o leaf node. */
+  return knode->type >= LIGHT_TREE_LEAF;
+}
+
+template<bool in_volume_segment>
+ccl_device void light_tree_to_local_space(KernelGlobals kg,
+                                          const int object_id,
+                                          ccl_private float3 &P,
+                                          ccl_private float3 &N_or_D,
+                                          ccl_private float &t)
+{
+  const int object_flag = kernel_data_fetch(object_flag, object_id);
+  if (!(object_flag & SD_OBJECT_TRANSFORM_APPLIED)) {
+#ifdef __OBJECT_MOTION__
+    Transform itfm;
+    object_fetch_transform_motion_test(kg, object_id, 0.5f, &itfm);
+#else
+    const Transform itfm = object_fetch_transform(kg, object_id, OBJECT_INVERSE_TRANSFORM);
+#endif
+    P = transform_point(&itfm, P);
+    if (in_volume_segment) {
+      /* Transform direction. */
+      float3 D_local = transform_direction(&itfm, N_or_D);
+      float scale;
+      N_or_D = normalize_len(D_local, &scale);
+
+      t *= scale;
+    }
+    else if (!is_zero(N_or_D)) {
+      /* Transform normal. */
+      const Transform tfm = object_fetch_transform(kg, object_id, OBJECT_TRANSFORM);
+      N_or_D = normalize(transform_direction_transposed(&tfm, N_or_D));
+    }
+  }
+}
+
 /* This is the general function for calculating the importance of either a cluster or an emitter.
  * Both of the specialized functions obtain the necessary data before calling this function. */
 template<bool in_volume_segment>
@@ -136,7 +190,8 @@ ccl_device void light_tree_importance(const float3 N_or_D,
     cos_min_outgoing_angle = 1.0f;
   }
   else if ((bcone.theta_o + bcone.theta_e > M_PI_F) ||
-           (cos_theta_minus_theta_u > cos(bcone.theta_o + bcone.theta_e))) {
+           (cos_theta_minus_theta_u > cos(bcone.theta_o + bcone.theta_e)))
+  {
     /* theta' = theta - theta_o - theta_u < theta_e */
     kernel_assert(
         (fast_acosf(cos_theta) - bcone.theta_o - fast_acosf(cos_theta_u) - bcone.theta_e) < 5e-4f);
@@ -165,7 +220,8 @@ ccl_device void light_tree_importance(const float3 N_or_D,
   float cos_max_outgoing_angle;
   const float cos_theta_plus_theta_u = cos_theta * cos_theta_u - sin_theta * sin_theta_u;
   if (bcone.theta_e - bcone.theta_o < 0 || cos_theta < 0 || cos_theta_u < 0 ||
-      cos_theta_plus_theta_u < cos(bcone.theta_e - bcone.theta_o)) {
+      cos_theta_plus_theta_u < cos(bcone.theta_e - bcone.theta_o))
+  {
     min_importance = 0.0f;
   }
   else {
@@ -184,9 +240,8 @@ ccl_device bool compute_emitter_centroid_and_dir(KernelGlobals kg,
                                                  ccl_private float3 &centroid,
                                                  ccl_private packed_float3 &dir)
 {
-  const int prim_id = kemitter->prim;
-  if (prim_id < 0) {
-    const ccl_global KernelLight *klight = &kernel_data_fetch(lights, ~prim_id);
+  if (is_light(kemitter)) {
+    const ccl_global KernelLight *klight = &kernel_data_fetch(lights, ~(kemitter->light.id));
     centroid = klight->co;
 
     switch (klight->type) {
@@ -213,19 +268,22 @@ ccl_device bool compute_emitter_centroid_and_dir(KernelGlobals kg,
     }
   }
   else {
+    kernel_assert(is_triangle(kemitter));
     const int object = kemitter->mesh_light.object_id;
     float3 vertices[3];
-    triangle_world_space_vertices(kg, object, prim_id, -1.0f, vertices);
+    triangle_vertices(kg, kemitter->triangle.id, vertices);
     centroid = (vertices[0] + vertices[1] + vertices[2]) / 3.0f;
 
-    const bool is_front_only = (kemitter->emission_sampling == EMISSION_SAMPLING_FRONT);
-    const bool is_back_only = (kemitter->emission_sampling == EMISSION_SAMPLING_BACK);
+    const bool is_front_only = (kemitter->triangle.emission_sampling == EMISSION_SAMPLING_FRONT);
+    const bool is_back_only = (kemitter->triangle.emission_sampling == EMISSION_SAMPLING_BACK);
     if (is_front_only || is_back_only) {
       dir = safe_normalize(cross(vertices[1] - vertices[0], vertices[2] - vertices[0]));
       if (is_back_only) {
         dir = -dir;
       }
-      if (kernel_data_fetch(object_flag, object) & SD_OBJECT_NEGATIVE_SCALE) {
+      const int object_flag = kernel_data_fetch(object_flag, object);
+      if ((object_flag & SD_OBJECT_TRANSFORM_APPLIED) && (object_flag & SD_OBJECT_NEGATIVE_SCALE))
+      {
         dir = -dir;
       }
     }
@@ -237,6 +295,75 @@ ccl_device bool compute_emitter_centroid_and_dir(KernelGlobals kg,
   return true;
 }
 
+template<bool in_volume_segment>
+ccl_device void light_tree_node_importance(KernelGlobals kg,
+                                           const float3 P,
+                                           const float3 N_or_D,
+                                           const float t,
+                                           const bool has_transmission,
+                                           const ccl_global KernelLightTreeNode *knode,
+                                           ccl_private float &max_importance,
+                                           ccl_private float &min_importance)
+{
+  const BoundingCone bcone = knode->bcone;
+  const BoundingBox bbox = knode->bbox;
+
+  float3 point_to_centroid;
+  float cos_theta_u;
+  float distance;
+  if (knode->type == LIGHT_TREE_DISTANT) {
+    if (in_volume_segment) {
+      return;
+    }
+    point_to_centroid = -bcone.axis;
+    cos_theta_u = fast_cosf(bcone.theta_o + bcone.theta_e);
+    distance = 1.0f;
+  }
+  else {
+    const float3 centroid = 0.5f * (bbox.min + bbox.max);
+
+    if (in_volume_segment) {
+      const float3 D = N_or_D;
+      const float3 closest_point = P + dot(centroid - P, D) * D;
+      /* Minimal distance of the ray to the cluster. */
+      distance = len(centroid - closest_point);
+      point_to_centroid = -compute_v(centroid, P, D, bcone.axis, t);
+      cos_theta_u = light_tree_cos_bounding_box_angle(bbox, closest_point, point_to_centroid);
+    }
+    else {
+      const float3 N = N_or_D;
+      const float3 bbox_extent = bbox.max - centroid;
+      const bool bbox_is_visible = has_transmission |
+                                   (dot(N, centroid - P) + dot(fabs(N), fabs(bbox_extent)) > 0);
+
+      /* If the node is guaranteed to be behind the surface we're sampling, and the surface is
+       * opaque, then we can give the node an importance of 0 as it contributes nothing to the
+       * surface. */
+      if (!bbox_is_visible) {
+        return;
+      }
+
+      point_to_centroid = normalize_len(centroid - P, &distance);
+      cos_theta_u = light_tree_cos_bounding_box_angle(bbox, P, point_to_centroid);
+    }
+    /* Clamp distance to half the radius of the cluster when splitting is disabled. */
+    distance = fmaxf(0.5f * len(centroid - bbox.max), distance);
+  }
+  /* TODO: currently max_distance = min_distance, max_importance = min_importance for the
+   * nodes. Do we need better weights for complex scenes? */
+  light_tree_importance<in_volume_segment>(N_or_D,
+                                           has_transmission,
+                                           point_to_centroid,
+                                           cos_theta_u,
+                                           bcone,
+                                           distance,
+                                           distance,
+                                           t,
+                                           knode->energy,
+                                           max_importance,
+                                           min_importance);
+}
+
 template<bool in_volume_segment>
 ccl_device void light_tree_emitter_importance(KernelGlobals kg,
                                               const float3 P,
@@ -247,11 +374,21 @@ ccl_device void light_tree_emitter_importance(KernelGlobals kg,
                                               ccl_private float &max_importance,
                                               ccl_private float &min_importance)
 {
+  max_importance = 0.0f;
+  min_importance = 0.0f;
+
   const ccl_global KernelLightTreeEmitter *kemitter = &kernel_data_fetch(light_tree_emitters,
                                                                          emitter_index);
 
-  max_importance = 0.0f;
-  min_importance = 0.0f;
+  if (is_mesh(kemitter)) {
+    const ccl_global KernelLightTreeNode *knode = &kernel_data_fetch(light_tree_nodes,
+                                                                     kemitter->mesh.node_id);
+
+    light_tree_node_importance<in_volume_segment>(
+        kg, P, N_or_D, t, has_transmission, knode, max_importance, min_importance);
+    return;
+  }
+
   BoundingCone bcone;
   bcone.theta_o = kemitter->theta_o;
   bcone.theta_e = kemitter->theta_e;
@@ -259,13 +396,11 @@ ccl_device void light_tree_emitter_importance(KernelGlobals kg,
   float2 distance; /* distance.x = max_distance, distance.y = mix_distance */
   float3 centroid, point_to_centroid, P_c;
 
-  if (!compute_emitter_centroid_and_dir<in_volume_segment>(
-          kg, kemitter, P, centroid, bcone.axis)) {
+  if (!compute_emitter_centroid_and_dir<in_volume_segment>(kg, kemitter, P, centroid, bcone.axis))
+  {
     return;
   }
 
-  const int prim_id = kemitter->prim;
-
   if (in_volume_segment) {
     const float3 D = N_or_D;
     /* Closest point. */
@@ -279,9 +414,15 @@ ccl_device void light_tree_emitter_importance(KernelGlobals kg,
     P_c = P;
   }
 
+  /* Early out if the emitter is guaranteed to be invisible. */
   bool is_visible;
-  if (prim_id < 0) {
-    const ccl_global KernelLight *klight = &kernel_data_fetch(lights, ~prim_id);
+  if (is_triangle(kemitter)) {
+    is_visible = triangle_light_tree_parameters<in_volume_segment>(
+        kg, kemitter, centroid, P_c, N_or_D, bcone, cos_theta_u, distance, point_to_centroid);
+  }
+  else {
+    kernel_assert(is_light(kemitter));
+    const ccl_global KernelLight *klight = &kernel_data_fetch(lights, ~(kemitter->light.id));
     switch (klight->type) {
       /* Function templates only modifies cos_theta_u when in_volume_segment = true. */
       case LIGHT_SPOT:
@@ -309,10 +450,6 @@ ccl_device void light_tree_emitter_importance(KernelGlobals kg,
         return;
     }
   }
-  else { /* Mesh light. */
-    is_visible = triangle_light_tree_parameters<in_volume_segment>(
-        kg, kemitter, centroid, P_c, N_or_D, bcone, cos_theta_u, distance, point_to_centroid);
-  }
 
   is_visible |= has_transmission;
   if (!is_visible) {
@@ -333,81 +470,31 @@ ccl_device void light_tree_emitter_importance(KernelGlobals kg,
 }
 
 template<bool in_volume_segment>
-ccl_device void light_tree_node_importance(KernelGlobals kg,
-                                           const float3 P,
-                                           const float3 N_or_D,
-                                           const float t,
-                                           const bool has_transmission,
-                                           const ccl_global KernelLightTreeNode *knode,
-                                           ccl_private float &max_importance,
-                                           ccl_private float &min_importance)
+ccl_device void light_tree_child_importance(KernelGlobals kg,
+                                            const float3 P,
+                                            const float3 N_or_D,
+                                            const float t,
+                                            const bool has_transmission,
+                                            const ccl_global KernelLightTreeNode *knode,
+                                            ccl_private float &max_importance,
+                                            ccl_private float &min_importance)
 {
   max_importance = 0.0f;
   min_importance = 0.0f;
-  if (knode->num_prims == 1) {
-    /* At a leaf node with only one emitter. */
-    light_tree_emitter_importance<in_volume_segment>(
-        kg, P, N_or_D, t, has_transmission, -knode->child_index, max_importance, min_importance);
+
+  if (knode->num_emitters == 1) {
+    light_tree_emitter_importance<in_volume_segment>(kg,
+                                                     P,
+                                                     N_or_D,
+                                                     t,
+                                                     has_transmission,
+                                                     knode->leaf.first_emitter,
+                                                     max_importance,
+                                                     min_importance);
   }
-  else if (knode->num_prims != 0) {
-    const BoundingCone bcone = knode->bcone;
-    const BoundingBox bbox = knode->bbox;
-
-    float3 point_to_centroid;
-    float cos_theta_u;
-    float distance;
-    if (knode->bit_trail == 1) {
-      /* Distant light node. */
-      if (in_volume_segment) {
-        return;
-      }
-      point_to_centroid = -bcone.axis;
-      cos_theta_u = fast_cosf(bcone.theta_o);
-      distance = 1.0f;
-    }
-    else {
-      const float3 centroid = 0.5f * (bbox.min + bbox.max);
-
-      if (in_volume_segment) {
-        const float3 D = N_or_D;
-        const float3 closest_point = P + dot(centroid - P, D) * D;
-        /* Minimal distance of the ray to the cluster. */
-        distance = len(centroid - closest_point);
-        point_to_centroid = -compute_v(centroid, P, D, bcone.axis, t);
-        cos_theta_u = light_tree_cos_bounding_box_angle(bbox, closest_point, point_to_centroid);
-      }
-      else {
-        const float3 N = N_or_D;
-        const float3 bbox_extent = bbox.max - centroid;
-        const bool bbox_is_visible = has_transmission |
-                                     (dot(N, centroid - P) + dot(fabs(N), fabs(bbox_extent)) > 0);
-
-        /* If the node is guaranteed to be behind the surface we're sampling, and the surface is
-         * opaque, then we can give the node an importance of 0 as it contributes nothing to the
-         * surface. */
-        if (!bbox_is_visible) {
-          return;
-        }
-
-        point_to_centroid = normalize_len(centroid - P, &distance);
-        cos_theta_u = light_tree_cos_bounding_box_angle(bbox, P, point_to_centroid);
-      }
-      /* Clamp distance to half the radius of the cluster when splitting is disabled. */
-      distance = fmaxf(0.5f * len(centroid - bbox.max), distance);
-    }
-    /* TODO: currently max_distance = min_distance, max_importance = min_importance for the
-     * nodes. Do we need better weights for complex scenes? */
-    light_tree_importance<in_volume_segment>(N_or_D,
-                                             has_transmission,
-                                             point_to_centroid,
-                                             cos_theta_u,
-                                             bcone,
-                                             distance,
-                                             distance,
-                                             t,
-                                             knode->energy,
-                                             max_importance,
-                                             min_importance);
+  else if (knode->num_emitters != 0) {
+    light_tree_node_importance<in_volume_segment>(
+        kg, P, N_or_D, t, has_transmission, knode, max_importance, min_importance);
   }
 }
 
@@ -422,6 +509,19 @@ ccl_device void sample_resevoir(const int current_index,
     return;
   }
   total_weight += current_weight;
+
+  /* When `-ffast-math` is used it is possible that the threshold is almost 1 but not quite.
+   * For this case we check the first assignment explicitly (instead of relying on the threshold to
+   * be 1, giving it certain probability). */
+  if (selected_index == -1) {
+    selected_index = current_index;
+    selected_weight = current_weight;
+    /* The threshold is expected to be 1 in this case with strict mathematics, so no need to divide
+     * the rand. In fact, division in such case could lead the rand to exceed 1 because of division
+     * by something smaller than 1. */
+    return;
+  }
+
   float thresh = current_weight / total_weight;
   if (rand <= thresh) {
     selected_index = current_index;
@@ -431,8 +531,10 @@ ccl_device void sample_resevoir(const int current_index,
   else {
     rand = (rand - thresh) / (1.0f - thresh);
   }
-  kernel_assert(rand >= 0.0f && rand <= 1.0f);
-  return;
+
+  /* Ensure the `rand` is always within 0..1 range, which could be violated above when
+   * `-ffast-math` is used. */
+  rand = saturatef(rand);
 }
 
 /* Pick an emitter from a leaf node using resevoir sampling, keep two reservoirs for upper and
@@ -440,26 +542,30 @@ ccl_device void sample_resevoir(const int current_index,
 template<bool in_volume_segment>
 ccl_device int light_tree_cluster_select_emitter(KernelGlobals kg,
                                                  ccl_private float &rand,
-                                                 const float3 P,
-                                                 const float3 N_or_D,
-                                                 const float t,
+                                                 ccl_private float3 &P,
+                                                 ccl_private float3 &N_or_D,
+                                                 ccl_private float &t,
                                                  const bool has_transmission,
-                                                 const ccl_global KernelLightTreeNode *knode,
+                                                 ccl_private int *node_index,
                                                  ccl_private float *pdf_factor)
 {
   float selected_importance[2] = {0.0f, 0.0f};
   float total_importance[2] = {0.0f, 0.0f};
   int selected_index = -1;
+  const ccl_global KernelLightTreeNode *knode = &kernel_data_fetch(light_tree_nodes, *node_index);
+  *node_index = -1;
 
   /* Mark emitters with zero importance. Used for resevoir when total minimum importance = 0. */
-  kernel_assert(knode->num_prims <= sizeof(uint) * 8);
+  kernel_assert(knode->num_emitters <= sizeof(uint) * 8);
   uint has_importance = 0;
 
   const bool sample_max = (rand > 0.5f); /* Sampling using the maximum importance. */
-  rand = rand * 2.0f - float(sample_max);
+  if (knode->num_emitters > 1) {
+    rand = rand * 2.0f - float(sample_max);
+  }
 
-  for (int i = 0; i < knode->num_prims; i++) {
-    int current_index = -knode->child_index + i;
+  for (int i = 0; i < knode->num_emitters; i++) {
+    int current_index = knode->leaf.first_emitter + i;
     /* maximum importance = importance[0], minimum importance = importance[1] */
     float importance[2];
     light_tree_emitter_importance<in_volume_segment>(
@@ -491,8 +597,8 @@ ccl_device int light_tree_cluster_select_emitter(KernelGlobals kg,
     }
     else {
       selected_index = -1;
-      for (int i = 0; i < knode->num_prims; i++) {
-        int current_index = -knode->child_index + i;
+      for (int i = 0; i < knode->num_emitters; i++) {
+        int current_index = knode->inner.right_child + i;
         sample_resevoir(current_index,
                         float(has_importance & 1),
                         selected_index,
@@ -508,8 +614,24 @@ ccl_device int light_tree_cluster_select_emitter(KernelGlobals kg,
     }
   }
 
-  *pdf_factor = 0.5f * (selected_importance[0] / total_importance[0] +
-                        selected_importance[1] / total_importance[1]);
+  *pdf_factor *= 0.5f * (selected_importance[0] / total_importance[0] +
+                         selected_importance[1] / total_importance[1]);
+
+  const ccl_global KernelLightTreeEmitter *kemitter = &kernel_data_fetch(light_tree_emitters,
+                                                                         selected_index);
+
+  if (is_mesh(kemitter)) {
+    /* Transform ray from world to local space. */
+    light_tree_to_local_space<in_volume_segment>(kg, kemitter->mesh.object_id, P, N_or_D, t);
+
+    *node_index = kemitter->mesh.node_id;
+    const ccl_global KernelLightTreeNode *knode = &kernel_data_fetch(light_tree_nodes,
+                                                                     *node_index);
+    if (knode->type == LIGHT_TREE_INSTANCE) {
+      /* Switch to the node with the subtree. */
+      *node_index = knode->instance.reference;
+    }
+  }
 
   return selected_index;
 }
@@ -528,9 +650,9 @@ ccl_device bool get_left_probability(KernelGlobals kg,
   const ccl_global KernelLightTreeNode *right = &kernel_data_fetch(light_tree_nodes, right_index);
 
   float min_left_importance, max_left_importance, min_right_importance, max_right_importance;
-  light_tree_node_importance<in_volume_segment>(
+  light_tree_child_importance<in_volume_segment>(
       kg, P, N_or_D, t, has_transmission, left, max_left_importance, min_left_importance);
-  light_tree_node_importance<in_volume_segment>(
+  light_tree_child_importance<in_volume_segment>(
       kg, P, N_or_D, t, has_transmission, right, max_right_importance, min_right_importance);
 
   const float total_max_importance = max_left_importance + max_right_importance;
@@ -556,8 +678,8 @@ ccl_device_noinline bool light_tree_sample(KernelGlobals kg,
                                            const float randv,
                                            const float time,
                                            const float3 P,
-                                           const float3 N_or_D,
-                                           const float t,
+                                           float3 N_or_D,
+                                           float t,
                                            const int shader_flags,
                                            const int bounce,
                                            const uint32_t path_flag,
@@ -571,28 +693,39 @@ ccl_device_noinline bool light_tree_sample(KernelGlobals kg,
   float pdf_leaf = 1.0f;
   float pdf_selection = 1.0f;
   int selected_emitter = -1;
-
+  int object = 0;
   int node_index = 0; /* Root node. */
 
+  float3 local_P = P;
+
   /* Traverse the light tree until a leaf node is reached. */
   while (true) {
     const ccl_global KernelLightTreeNode *knode = &kernel_data_fetch(light_tree_nodes, node_index);
 
-    if (knode->child_index <= 0) {
+    if (is_leaf(knode)) {
       /* At a leaf node, we pick an emitter. */
       selected_emitter = light_tree_cluster_select_emitter<in_volume_segment>(
-          kg, randn, P, N_or_D, t, has_transmission, knode, &pdf_selection);
-      break;
+          kg, randn, local_P, N_or_D, t, has_transmission, &node_index, &pdf_selection);
+
+      if (node_index < 0) {
+        break;
+      }
+      else {
+        /* Continue with the picked mesh light. */
+        object = kernel_data_fetch(light_tree_emitters, selected_emitter).mesh.object_id;
+        continue;
+      }
     }
 
     /* At an interior node, the left child is directly after the parent, while the right child is
      * stored as the child index. */
     const int left_index = node_index + 1;
-    const int right_index = knode->child_index;
+    const int right_index = knode->inner.right_child;
 
     float left_prob;
     if (!get_left_probability<in_volume_segment>(
-            kg, P, N_or_D, t, has_transmission, left_index, right_index, left_prob)) {
+            kg, local_P, N_or_D, t, has_transmission, left_index, right_index, left_prob))
+    {
       return false; /* Both child nodes have zero importance. */
     }
 
@@ -610,42 +743,101 @@ ccl_device_noinline bool light_tree_sample(KernelGlobals kg,
   pdf_selection *= pdf_leaf;
 
   return light_sample<in_volume_segment>(
-      kg, randu, randv, time, P, bounce, path_flag, selected_emitter, pdf_selection, ls);
+      kg, randu, randv, time, P, bounce, path_flag, selected_emitter, object, pdf_selection, ls);
 }
 
 /* We need to be able to find the probability of selecting a given light for MIS. */
 ccl_device float light_tree_pdf(
-    KernelGlobals kg, const float3 P, const float3 N, const int path_flag, const int prim)
+    KernelGlobals kg, float3 P, float3 N, const int path_flag, const int object, const uint target)
 {
   const bool has_transmission = (path_flag & PATH_RAY_MIS_HAD_TRANSMISSION);
-  /* Target emitter info. */
-  const int target_emitter = (prim >= 0) ? kernel_data_fetch(triangle_to_tree, prim) :
-                                           kernel_data_fetch(light_to_tree, ~prim);
-  ccl_global const KernelLightTreeEmitter *kemitter = &kernel_data_fetch(light_tree_emitters,
-                                                                         target_emitter);
-  const int target_leaf = kemitter->parent_index;
-  ccl_global const KernelLightTreeNode *kleaf = &kernel_data_fetch(light_tree_nodes, target_leaf);
-  uint bit_trail = kleaf->bit_trail;
 
-  int node_index = 0; /* Root node. */
+  ccl_global const KernelLightTreeEmitter *kemitter = &kernel_data_fetch(light_tree_emitters,
+                                                                         target);
+  int root_index;
+  uint bit_trail, target_emitter;
+
+  if (is_triangle(kemitter)) {
+    /* If the target is an emissive triangle, first traverse the top level tree to find the mesh
+     * light emitter, then traverse the subtree. */
+    target_emitter = kernel_data_fetch(object_to_tree, object);
+    ccl_global const KernelLightTreeEmitter *kmesh = &kernel_data_fetch(light_tree_emitters,
+                                                                        target_emitter);
+    root_index = kmesh->mesh.node_id;
+    ccl_global const KernelLightTreeNode *kroot = &kernel_data_fetch(light_tree_nodes, root_index);
+    bit_trail = kroot->bit_trail;
+
+    if (kroot->type == LIGHT_TREE_INSTANCE) {
+      root_index = kroot->instance.reference;
+    }
+  }
+  else {
+    root_index = 0;
+    bit_trail = kemitter->bit_trail;
+    target_emitter = target;
+  }
 
   float pdf = 1.0f;
+  int node_index = 0;
 
   /* Traverse the light tree until we reach the target leaf node. */
   while (true) {
     const ccl_global KernelLightTreeNode *knode = &kernel_data_fetch(light_tree_nodes, node_index);
 
-    if (knode->child_index <= 0) {
-      break;
+    if (is_leaf(knode)) {
+      /* Iterate through leaf node to find the probability of sampling the target emitter. */
+      float target_max_importance = 0.0f;
+      float target_min_importance = 0.0f;
+      float total_max_importance = 0.0f;
+      float total_min_importance = 0.0f;
+      int num_has_importance = 0;
+      for (int i = 0; i < knode->num_emitters; i++) {
+        const int emitter = knode->leaf.first_emitter + i;
+        float max_importance, min_importance;
+        light_tree_emitter_importance<false>(
+            kg, P, N, 0, has_transmission, emitter, max_importance, min_importance);
+        num_has_importance += (max_importance > 0);
+        if (emitter == target_emitter) {
+          target_max_importance = max_importance;
+          target_min_importance = min_importance;
+        }
+        total_max_importance += max_importance;
+        total_min_importance += min_importance;
+      }
+
+      if (target_max_importance > 0.0f) {
+        pdf *= 0.5f * (target_max_importance / total_max_importance +
+                       (total_min_importance > 0 ? target_min_importance / total_min_importance :
+                                                   1.0f / num_has_importance));
+      }
+      else {
+        return 0.0f;
+      }
+
+      if (root_index) {
+        /* Arrived at the mesh light. Continue with the subtree. */
+        float unused;
+        light_tree_to_local_space<false>(kg, object, P, N, unused);
+
+        node_index = root_index;
+        root_index = 0;
+        target_emitter = target;
+        bit_trail = kemitter->bit_trail;
+        continue;
+      }
+      else {
+        return pdf;
+      }
     }
 
     /* Interior node. */
     const int left_index = node_index + 1;
-    const int right_index = knode->child_index;
+    const int right_index = knode->inner.right_child;
 
     float left_prob;
     if (!get_left_probability<false>(
-            kg, P, N, 0, has_transmission, left_index, right_index, left_prob)) {
+            kg, P, N, 0, has_transmission, left_index, right_index, left_prob))
+    {
       return 0.0f;
     }
 
@@ -658,36 +850,6 @@ ccl_device float light_tree_pdf(
       return 0.0f;
     }
   }
-
-  kernel_assert(node_index == target_leaf);
-
-  /* Iterate through leaf node to find the probability of sampling the target emitter. */
-  float target_max_importance = 0.0f;
-  float target_min_importance = 0.0f;
-  float total_max_importance = 0.0f;
-  float total_min_importance = 0.0f;
-  int num_has_importance = 0;
-  for (int i = 0; i < kleaf->num_prims; i++) {
-    const int emitter = -kleaf->child_index + i;
-    float max_importance, min_importance;
-    light_tree_emitter_importance<false>(
-        kg, P, N, 0, has_transmission, emitter, max_importance, min_importance);
-    num_has_importance += (max_importance > 0);
-    if (emitter == target_emitter) {
-      target_max_importance = max_importance;
-      target_min_importance = min_importance;
-    }
-    total_max_importance += max_importance;
-    total_min_importance += min_importance;
-  }
-
-  if (target_max_importance > 0.0f) {
-    return pdf * 0.5f *
-           (target_max_importance / total_max_importance +
-            (total_min_importance > 0 ? target_min_importance / total_min_importance :
-                                        1.0f / num_has_importance));
-  }
-  return 0.0f;
 }
 
 CCL_NAMESPACE_END
