@@ -1,27 +1,9 @@
-# <pep8-80 compliant>
-
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 __author__ = "Nutti <nutti.metro@gmail.com>"
 __status__ = "production"
-__version__ = "6.3"
-__date__ = "10 Aug 2020"
+__version__ = "6.6"
+__date__ = "22 Apr 2022"
 
 from collections import defaultdict
 from pprint import pprint
@@ -33,6 +15,7 @@ from mathutils import Vector
 import bmesh
 
 from .utils import compatibility as compat
+from .utils.graph import Graph, Node
 
 
 __DEBUG_MODE = False
@@ -44,6 +27,14 @@ def is_console_mode():
     return os.environ["MUV_CONSOLE_MODE"] == "true"
 
 
+def is_valid_space(context, allowed_spaces):
+    for area in context.screen.areas:
+        for space in area.spaces:
+            if space.type in allowed_spaces:
+                return True
+    return False
+
+
 def is_debug_mode():
     return __DEBUG_MODE
 
@@ -294,6 +285,30 @@ def get_island_info(obj, only_selected=True):
     return get_island_info_from_bmesh(bm, only_selected)
 
 
+# Return island info.
+#
+# Format:
+#
+# [
+#   {
+#     faces: [
+#       {
+#         face: BMFace
+#         max_uv: Vector (2D)
+#         min_uv: Vector (2D)
+#         ave_uv: Vector (2D)
+#       },
+#       ...
+#     ]
+#     center: Vector (2D)
+#     size: Vector (2D)
+#     num_uv: int
+#     group: int
+#     max: Vector (2D)
+#     min: Vector (2D)
+#   },
+#   ...
+# ]
 def get_island_info_from_bmesh(bm, only_selected=True):
     if not bm.loops.layers.uv:
         return None
@@ -325,7 +340,7 @@ def get_uvimg_editor_board_size(area):
     return (255.0, 255.0)
 
 
-def calc_polygon_2d_area(points):
+def calc_tris_2d_area(points):
     area = 0.0
     for i, p1 in enumerate(points):
         p2 = points[(i + 1) % len(points)]
@@ -337,7 +352,7 @@ def calc_polygon_2d_area(points):
     return fabs(0.5 * area)
 
 
-def calc_polygon_3d_area(points):
+def calc_tris_3d_area(points):
     area = 0.0
     for i, p1 in enumerate(points):
         p2 = points[(i + 1) % len(points)]
@@ -387,6 +402,23 @@ def get_faces_list(bm, method, only_selected):
     return faces_list
 
 
+def measure_all_faces_mesh_area(bm):
+    if compat.check_version(2, 80, 0) >= 0:
+        triangle_loops = bm.calc_loop_triangles()
+    else:
+        triangle_loops = bm.calc_tessface()
+
+    areas = {face: 0.0 for face in bm.faces}
+
+    for loops in triangle_loops:
+        face = loops[0].face
+        area = areas[face]
+        area += calc_tris_3d_area([l.vert.co for l in loops])
+        areas[face] = area
+
+    return areas
+
+
 def measure_mesh_area(obj, calc_method, only_selected):
     bm = bmesh.from_edit_mesh(obj.data)
     if check_version(2, 73, 0) >= 0:
@@ -398,17 +430,18 @@ def measure_mesh_area(obj, calc_method, only_selected):
 
     areas = []
     for faces in faces_list:
-        areas.append(measure_mesh_area_from_faces(faces))
+        areas.append(measure_mesh_area_from_faces(bm, faces))
 
     return areas
 
 
-def measure_mesh_area_from_faces(faces):
+def measure_mesh_area_from_faces(bm, faces):
+    face_areas = measure_all_faces_mesh_area(bm)
+
     mesh_area = 0.0
     for f in faces:
-        verts = [l.vert.co for l in f.loops]
-        f_mesh_area = calc_polygon_3d_area(verts)
-        mesh_area = mesh_area + f_mesh_area
+        if f in face_areas:
+            mesh_area += face_areas[f]
 
     return mesh_area
 
@@ -422,23 +455,30 @@ def find_texture_layer(bm):
     return bm.faces.layers.tex.verify()
 
 
+def find_texture_nodes_from_material(mtrl):
+    nodes = []
+    if not mtrl.node_tree:
+        return nodes
+    for node in mtrl.node_tree.nodes:
+        tex_node_types = [
+            'TEX_ENVIRONMENT',
+            'TEX_IMAGE',
+        ]
+        if node.type not in tex_node_types:
+            continue
+        if not node.image:
+            continue
+        nodes.append(node)
+
+    return nodes
+
+
 def find_texture_nodes(obj):
     nodes = []
-    for mat in obj.material_slots:
-        if not mat.material:
+    for slot in obj.material_slots:
+        if not slot.material:
             continue
-        if not mat.material.node_tree:
-            continue
-        for node in mat.material.node_tree.nodes:
-            tex_node_types = [
-                'TEX_ENVIRONMENT',
-                'TEX_IMAGE',
-            ]
-            if node.type not in tex_node_types:
-                continue
-            if not node.image:
-                continue
-            nodes.append(node)
+        nodes.extend(find_texture_nodes_from_material(slot.material))
 
     return nodes
 
@@ -471,12 +511,34 @@ def find_images(obj, face=None, tex_layer=None):
     return images
 
 
-def measure_uv_area_from_faces(obj, faces, uv_layer, tex_layer,
+def measure_all_faces_uv_area(bm, uv_layer):
+    if compat.check_version(2, 80, 0) >= 0:
+        triangle_loops = bm.calc_loop_triangles()
+    else:
+        triangle_loops = bm.calc_tessface()
+
+    areas = {face: 0.0 for face in bm.faces}
+
+    for loops in triangle_loops:
+        face = loops[0].face
+        area = areas[face]
+        area += calc_tris_2d_area([l[uv_layer].uv for l in loops])
+        areas[face] = area
+
+    return areas
+
+
+def measure_uv_area_from_faces(obj, bm, faces, uv_layer, tex_layer,
                                tex_selection_method, tex_size):
+
+    face_areas = measure_all_faces_uv_area(bm, uv_layer)
+
     uv_area = 0.0
     for f in faces:
-        uvs = [l[uv_layer].uv for l in f.loops]
-        f_uv_area = calc_polygon_2d_area(uvs)
+        if f not in face_areas:
+            continue
+
+        f_uv_area = face_areas[f]
 
         # user specified
         if tex_selection_method == 'USER_SPECIFIED' and tex_size is not None:
@@ -532,8 +594,8 @@ def measure_uv_area_from_faces(obj, faces, uv_layer, tex_layer,
     return uv_area
 
 
-def measure_uv_area(obj, calc_method, tex_selection_method, tex_size,
-                    only_selected):
+def measure_uv_area(obj, calc_method, tex_selection_method,
+                    tex_size, only_selected):
     bm = bmesh.from_edit_mesh(obj.data)
     if check_version(2, 73, 0) >= 0:
         bm.verts.ensure_lookup_table()
@@ -550,7 +612,8 @@ def measure_uv_area(obj, calc_method, tex_selection_method, tex_size,
     uv_areas = []
     for faces in faces_list:
         uv_area = measure_uv_area_from_faces(
-            obj, faces, uv_layer, tex_layer, tex_selection_method, tex_size)
+            obj, bm, faces, uv_layer, tex_layer,
+            tex_selection_method, tex_size)
         if uv_area is None:
             return None
         uv_areas.append(uv_area)
@@ -931,7 +994,8 @@ class RingBuffer:
 
 # clip: reference polygon
 # subject: tested polygon
-def __do_weiler_atherton_cliping(clip_uvs, subject_uvs, mode):
+def __do_weiler_atherton_cliping(clip_uvs, subject_uvs, mode,
+                                 same_polygon_threshold):
 
     clip_uvs = RingBuffer(clip_uvs)
     if __is_polygon_flipped(clip_uvs):
@@ -946,7 +1010,7 @@ def __do_weiler_atherton_cliping(clip_uvs, subject_uvs, mode):
     debug_print(subject_uvs)
 
     # check if clip and subject is overlapped completely
-    if __is_polygon_same(clip_uvs, subject_uvs):
+    if __is_polygon_same(clip_uvs, subject_uvs, same_polygon_threshold):
         polygons = [subject_uvs.as_list()]
         debug_print("===== Polygons Overlapped Completely =====")
         debug_print(polygons)
@@ -1143,12 +1207,22 @@ def __is_polygon_flipped(points):
 
 
 def __is_point_in_polygon(point, subject_points):
+    """Return true when point is inside of the polygon by using
+    'Crossing number algorithm'.
+    """
+
     count = 0
     for i in range(len(subject_points)):
         uv_start1 = subject_points.get(i)
         uv_end1 = subject_points.get(i + 1)
         uv_start2 = point
         uv_end2 = Vector((1000000.0, point.y))
+
+        # If the point exactly matches to the point of the polygon,
+        # this point is not in polygon.
+        if uv_start1.x == uv_start2.x and uv_start1.y == uv_start2.y:
+            return False
+
         intersected, _ = __is_segment_intersect(uv_start1, uv_end1,
                                                 uv_start2, uv_end2)
         if intersected:
@@ -1166,26 +1240,46 @@ def __is_points_in_polygon(points, subject_points):
     return True
 
 
-def get_overlapped_uv_info(bm_list, faces_list, uv_layer_list, mode):
+def get_uv_editable_objects(context):
+    if compat.check_version(2, 80, 0) < 0:
+        objs = []
+    else:
+        objs = [o for o in bpy.data.objects
+                if compat.get_object_select(o) and o.type == 'MESH']
+
+    ob = context.active_object
+    if ob is not None:
+        objs.append(ob)
+
+    objs = list(set(objs))
+    return objs
+
+
+def get_overlapped_uv_info(bm_list, faces_list, uv_layer_list,
+                           mode, same_polygon_threshold=0.0000001):
     # at first, check island overlapped
     isl = []
     for bm, uv_layer, faces in zip(bm_list, uv_layer_list, faces_list):
         info = get_island_info_from_faces(bm, faces, uv_layer)
-        isl.extend([(i, uv_layer) for i in info])
+        isl.extend([(i, uv_layer, bm) for i in info])
 
     overlapped_isl_pairs = []
     overlapped_uv_layer_pairs = []
-    for i, (i1, uv_layer_1) in enumerate(isl):
-        for i2, uv_layer_2 in isl[i + 1:]:
+    overlapped_bm_paris = []
+    for i, (i1, uv_layer_1, bm_1) in enumerate(isl):
+        for i2, uv_layer_2, bm_2 in isl[i + 1:]:
             if (i1["max"].x < i2["min"].x) or (i2["max"].x < i1["min"].x) or \
                (i1["max"].y < i2["min"].y) or (i2["max"].y < i1["min"].y):
                 continue
             overlapped_isl_pairs.append([i1, i2])
             overlapped_uv_layer_pairs.append([uv_layer_1, uv_layer_2])
+            overlapped_bm_paris.append([bm_1, bm_2])
 
-    # next, check polygon overlapped
+    # check polygon overlapped (inter UV islands)
     overlapped_uvs = []
-    for oip, uvlp in zip(overlapped_isl_pairs, overlapped_uv_layer_pairs):
+    for oip, uvlp, bmp in zip(overlapped_isl_pairs,
+                              overlapped_uv_layer_pairs,
+                              overlapped_bm_paris):
         for clip in oip[0]["faces"]:
             f_clip = clip["face"]
             clip_uvs = [l[uvlp[0]].uv.copy() for l in f_clip.loops]
@@ -1201,28 +1295,66 @@ def get_overlapped_uv_info(bm_list, faces_list, uv_layer_list, mode):
 
                 subject_uvs = [l[uvlp[1]].uv.copy() for l in f_subject.loops]
                 # slow operation, apply Weiler-Atherton cliping algorithm
-                result, polygons = __do_weiler_atherton_cliping(clip_uvs,
-                                                                subject_uvs,
-                                                                mode)
+                result, polygons = \
+                    __do_weiler_atherton_cliping(clip_uvs, subject_uvs,
+                                                 mode, same_polygon_threshold)
                 if result:
-                    overlapped_uvs.append({"clip_face": f_clip,
+                    overlapped_uvs.append({"clip_bmesh": bmp[0],
+                                           "subject_bmesh": bmp[1],
+                                           "clip_face": f_clip,
                                            "subject_face": f_subject,
                                            "clip_uv_layer": uvlp[0],
                                            "subject_uv_layer": uvlp[1],
                                            "subject_uvs": subject_uvs,
                                            "polygons": polygons})
 
+    # check polygon overlapped (intra UV island)
+    for info, uv_layer, bm in isl:
+        for i in range(len(info["faces"])):
+            clip = info["faces"][i]
+            f_clip = clip["face"]
+            clip_uvs = [l[uv_layer].uv.copy() for l in f_clip.loops]
+            for j in range(len(info["faces"])):
+                if j <= i:
+                    continue
+
+                subject = info["faces"][j]
+                f_subject = subject["face"]
+
+                # fast operation, apply bounding box algorithm
+                if (clip["max_uv"].x < subject["min_uv"].x) or \
+                   (subject["max_uv"].x < clip["min_uv"].x) or \
+                   (clip["max_uv"].y < subject["min_uv"].y) or \
+                   (subject["max_uv"].y < clip["min_uv"].y):
+                    continue
+
+                subject_uvs = [l[uv_layer].uv.copy() for l in f_subject.loops]
+                # slow operation, apply Weiler-Atherton cliping algorithm
+                result, polygons = \
+                    __do_weiler_atherton_cliping(clip_uvs, subject_uvs,
+                                                 mode, same_polygon_threshold)
+                if result:
+                    overlapped_uvs.append({"clip_bmesh": bm,
+                                           "subject_bmesh": bm,
+                                           "clip_face": f_clip,
+                                           "subject_face": f_subject,
+                                           "clip_uv_layer": uv_layer,
+                                           "subject_uv_layer": uv_layer,
+                                           "subject_uvs": subject_uvs,
+                                           "polygons": polygons})
+
     return overlapped_uvs
 
 
-def get_flipped_uv_info(faces_list, uv_layer_list):
+def get_flipped_uv_info(bm_list, faces_list, uv_layer_list):
     flipped_uvs = []
-    for faces, uv_layer in zip(faces_list, uv_layer_list):
+    for bm, faces, uv_layer in zip(bm_list, faces_list, uv_layer_list):
         for f in faces:
             polygon = RingBuffer([l[uv_layer].uv.copy() for l in f.loops])
             if __is_polygon_flipped(polygon):
                 uvs = [l[uv_layer].uv.copy() for l in f.loops]
-                flipped_uvs.append({"face": f,
+                flipped_uvs.append({"bmesh": bm,
+                                    "face": f,
                                     "uv_layer": uv_layer,
                                     "uvs": uvs,
                                     "polygons": [polygon.as_list()]})
@@ -1230,7 +1362,7 @@ def get_flipped_uv_info(faces_list, uv_layer_list):
     return flipped_uvs
 
 
-def __is_polygon_same(points1, points2):
+def __is_polygon_same(points1, points2, threshold):
     if len(points1) != len(points2):
         return False
 
@@ -1240,10 +1372,71 @@ def __is_polygon_same(points1, points2):
     for p1 in pts1:
         for p2 in pts2:
             diff = p2 - p1
-            if diff.length < 0.0000001:
+            if diff.length < threshold:
                 pts2.remove(p2)
                 break
         else:
             return False
 
     return True
+
+
+def _is_uv_loop_connected(l1, l2, uv_layer):
+    uv1 = l1[uv_layer].uv
+    uv2 = l2[uv_layer].uv
+    return uv1.x == uv2.x and uv1.y == uv2.y
+
+
+def create_uv_graph(loops, uv_layer):
+    # For looking up faster.
+    loop_index_to_loop = {}     # { loop index: loop }
+    for l in loops:
+        loop_index_to_loop[l.index] = l
+
+    # Setup relationship between uv_vert and loops.
+    # uv_vert is a representative of the loops which shares same
+    # UV coordinate.
+    uv_vert_to_loops = {}   # { uv_vert: loops belonged to uv_vert }
+    loop_to_uv_vert = {}    # { loop: uv_vert belonged to }
+    for l in loops:
+        found = False
+        for k in uv_vert_to_loops.keys():
+            if _is_uv_loop_connected(k, l, uv_layer):
+                uv_vert_to_loops[k].append(l)
+                loop_to_uv_vert[l] = k
+                found = True
+                break
+        if not found:
+            uv_vert_to_loops[l] = [l]
+            loop_to_uv_vert[l] = l
+
+    # Collect adjacent uv_vert.
+    uv_adj_verts = {}       # { uv_vert: adj uv_vert list }
+    for v, vs in uv_vert_to_loops.items():
+        uv_adj_verts[v] = []
+        for ll in vs:
+            ln = ll.link_loop_next
+            lp = ll.link_loop_prev
+            uv_adj_verts[v].append(loop_to_uv_vert[ln])
+            uv_adj_verts[v].append(loop_to_uv_vert[lp])
+        uv_adj_verts[v] = list(set(uv_adj_verts[v]))
+
+    # Setup uv_vert graph.
+    graph = Graph()
+    for v in uv_adj_verts.keys():
+        graph.add_node(
+            Node(v.index, {"uv_vert": v, "loops": uv_vert_to_loops[v]})
+        )
+    edges = []
+    for v, adjs in uv_adj_verts.items():
+        n1 = graph.get_node(v.index)
+        for a in adjs:
+            n2 = graph.get_node(a.index)
+            edges.append(tuple(sorted((n1.key, n2.key))))
+    edges = list(set(edges))
+    for e in edges:
+        n1 = graph.get_node(e[0])
+        n2 = graph.get_node(e[1])
+        graph.add_edge(n1, n2)
+
+    return graph
