@@ -1,33 +1,15 @@
-# <pep8-80 compliant>
-
-# ##### BEGIN GPL LICENSE BLOCK #####
-#
-#  This program is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU General Public License
-#  as published by the Free Software Foundation; either version 2
-#  of the License, or (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software Foundation,
-#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-#
-# ##### END GPL LICENSE BLOCK #####
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 __author__ = "Nutti <nutti.metro@gmail.com>"
 __status__ = "production"
-__version__ = "6.4"
-__date__ = "23 Oct 2020"
+__version__ = "6.7"
+__date__ = "9 Sep 2022"
 
 import random
 from math import fabs
 
 import bpy
-from bpy.props import BoolProperty, EnumProperty
+from bpy.props import BoolProperty, EnumProperty, FloatProperty
 import bmesh
 
 from .. import common
@@ -35,13 +17,17 @@ from ..utils.bl_class_registry import BlClassRegistry
 from ..utils.property_class_registry import PropertyClassRegistry
 from ..utils import compatibility as compat
 
-if compat.check_version(2, 80, 0) >= 0:
-    from ..lib import bglx as bgl
-else:
-    import bgl
+import gpu
+from gpu_extras.batch import batch_for_shader
 
 
 def _is_valid_context(context):
+    # 'IMAGE_EDITOR' and 'VIEW_3D' space is allowed to execute.
+    # If 'View_3D' space is not allowed, you can't find option in Tool-Shelf
+    # after the execution
+    if not common.is_valid_space(context, ['IMAGE_EDITOR', 'VIEW_3D']):
+        return False
+
     objs = common.get_uv_editable_objects(context)
     if not objs:
         return False
@@ -50,12 +36,6 @@ def _is_valid_context(context):
     if context.object.mode != 'EDIT':
         return False
 
-    # 'IMAGE_EDITOR' and 'VIEW_3D' space is allowed to execute.
-    # If 'View_3D' space is not allowed, you can't find option in Tool-Shelf
-    # after the execution
-    if not common.is_valid_space(context, ['IMAGE_EDITOR', 'VIEW_3D']):
-        return False
-
     return True
 
 
@@ -64,6 +44,7 @@ def _update_uvinsp_info(context):
     props = sc.muv_props.uv_inspection
     objs = common.get_uv_editable_objects(context)
 
+    bm_to_obj = {}      # { Object: BMesh }
     bm_list = []
     uv_layer_list = []
     faces_list = []
@@ -77,13 +58,35 @@ def _update_uvinsp_info(context):
             sel_faces = [f for f in bm.faces]
         else:
             sel_faces = [f for f in bm.faces if f.select]
+        bm_to_obj[bm] = obj
         bm_list.append(bm)
         uv_layer_list.append(uv_layer)
         faces_list.append(sel_faces)
 
     props.overlapped_info = common.get_overlapped_uv_info(
-        bm_list, faces_list, uv_layer_list, sc.muv_uv_inspection_show_mode)
-    props.flipped_info = common.get_flipped_uv_info(faces_list, uv_layer_list)
+        bm_list, faces_list, uv_layer_list, sc.muv_uv_inspection_show_mode,
+        sc.muv_uv_inspection_same_polygon_threshold)
+    props.flipped_info = common.get_flipped_uv_info(
+        bm_list, faces_list, uv_layer_list)
+
+    if sc.muv_uv_inspection_display_in_v3d:
+        props.overlapped_info_for_v3d = {}
+        for info in props.overlapped_info:
+            bm = info["subject_bmesh"]
+            face = info["subject_face"]
+            obj = bm_to_obj[bm]
+            if obj not in props.overlapped_info_for_v3d:
+                props.overlapped_info_for_v3d[obj] = []
+            props.overlapped_info_for_v3d[obj].append(face.index)
+
+        props.filpped_info_for_v3d = {}
+        for info in props.flipped_info:
+            bm = info["bmesh"]
+            face = info["face"]
+            obj = bm_to_obj[bm]
+            if obj not in props.filpped_info_for_v3d:
+                props.filpped_info_for_v3d[obj] = []
+            props.filpped_info_for_v3d[obj].append(face.index)
 
 
 @PropertyClassRegistry()
@@ -95,6 +98,8 @@ class _Properties:
         class Props():
             overlapped_info = []
             flipped_info = []
+            overlapped_info_for_v3d = {}    # { Object: [face_indices] }
+            filpped_info_for_v3d = {}       # { Object: [face_indices] }
 
         scene.muv_props.uv_inspection = Props()
 
@@ -130,6 +135,11 @@ class _Properties:
             description="Show flipped UVs",
             default=False
         )
+        scene.muv_uv_inspection_display_in_v3d = BoolProperty(
+            name="Display View3D",
+            description="Display overlapped/flipped faces on View3D",
+            default=True
+        )
         scene.muv_uv_inspection_show_mode = EnumProperty(
             name="Mode",
             description="Show mode",
@@ -139,6 +149,14 @@ class _Properties:
             ],
             default='PART'
         )
+        scene.muv_uv_inspection_same_polygon_threshold = FloatProperty(
+            name="Same Polygon Threshold",
+            description="Threshold to distinguish same polygons",
+            default=0.00001,
+            min=0.00001,
+            max=0.01,
+            step=0.00001
+        )
 
     @classmethod
     def del_props(cls, scene):
@@ -147,7 +165,9 @@ class _Properties:
         del scene.muv_uv_inspection_show
         del scene.muv_uv_inspection_show_overlapped
         del scene.muv_uv_inspection_show_flipped
+        del scene.muv_uv_inspection_display_in_v3d
         del scene.muv_uv_inspection_show_mode
+        del scene.muv_uv_inspection_same_polygon_threshold
 
 
 @BlClassRegistry()
@@ -162,6 +182,7 @@ class MUV_OT_UVInspection_Render(bpy.types.Operator):
     bl_label = "Overlapped/Flipped UV renderer"
 
     __handle = None
+    __handle_v3d = None
 
     @classmethod
     def poll(cls, context):
@@ -181,6 +202,11 @@ class MUV_OT_UVInspection_Render(bpy.types.Operator):
             MUV_OT_UVInspection_Render.draw, (obj, context),
             'WINDOW', 'POST_PIXEL')
 
+        sv3d = bpy.types.SpaceView3D
+        cls.__handle_v3d = sv3d.draw_handler_add(
+            MUV_OT_UVInspection_Render.draw_v3d, (obj, context),
+            'WINDOW', 'POST_VIEW')
+
     @classmethod
     def handle_remove(cls):
         if cls.__handle is not None:
@@ -188,6 +214,59 @@ class MUV_OT_UVInspection_Render(bpy.types.Operator):
                 cls.__handle, 'WINDOW')
             cls.__handle = None
 
+        if cls.__handle_v3d is not None:
+            bpy.types.SpaceView3D.draw_handler_remove(
+                cls.__handle_v3d, 'WINDOW')
+
+    @staticmethod
+    def draw_v3d(_, context):
+        sc = context.scene
+        props = sc.muv_props.uv_inspection
+        user_prefs = compat.get_user_preferences(context)
+        prefs = user_prefs.addons["magic_uv"].preferences
+
+        if not MUV_OT_UVInspection_Render.is_running(context):
+            return
+
+        if not sc.muv_uv_inspection_display_in_v3d:
+            return
+
+        # OpenGL configuration.
+        gpu.state.blend_set('ALPHA')
+        gpu.state.depth_test_set('LESS_EQUAL')
+
+        shader = gpu.shader.from_builtin("UNIFORM_COLOR")
+        shader.bind()
+
+        # Render faces whose UV is overlapped.
+        if sc.muv_uv_inspection_show_overlapped:
+            shader.uniform_float("color", prefs.uv_inspection_overlapped_color_for_v3d)
+
+            for obj, findices in props.overlapped_info_for_v3d.items():
+                world_mat = obj.matrix_world
+                bm = bmesh.from_edit_mesh(obj.data)
+
+                for fidx in findices:
+                    coords = [compat.matmul(world_mat, l.vert.co) for l in bm.faces[fidx].loops]
+                    batch = batch_for_shader(shader, 'TRI_FAN', {"pos": coords})
+                    batch.draw(shader)
+
+        # Render faces whose UV is flipped.
+        if sc.muv_uv_inspection_show_flipped:
+            shader.uniform_float("color", prefs.uv_inspection_flipped_color_for_v3d)
+
+            for obj, findices in props.filpped_info_for_v3d.items():
+                world_mat = obj.matrix_world
+                bm = bmesh.from_edit_mesh(obj.data)
+
+                for fidx in findices:
+                    coords = [compat.matmul(world_mat, l.vert.co) for l in bm.faces[fidx].loops]
+                    batch = batch_for_shader(shader, 'TRI_FAN', {"pos": coords})
+                    batch.draw(shader)
+
+        gpu.state.depth_test_set('NONE')
+        gpu.state.blend_set('NONE')
+
     @staticmethod
     def draw(_, context):
         sc = context.scene
@@ -199,53 +278,46 @@ class MUV_OT_UVInspection_Render(bpy.types.Operator):
             return
 
         # OpenGL configuration
-        bgl.glEnable(bgl.GL_BLEND)
+        gpu.state.blend_set('ALPHA')
+
+        shader = gpu.shader.from_builtin("UNIFORM_COLOR")
+        shader.bind()
 
         # render overlapped UV
         if sc.muv_uv_inspection_show_overlapped:
-            color = prefs.uv_inspection_overlapped_color
+            shader.uniform_float("color", prefs.uv_inspection_overlapped_color)
+
             for info in props.overlapped_info:
                 if sc.muv_uv_inspection_show_mode == 'PART':
                     for poly in info["polygons"]:
-                        bgl.glBegin(bgl.GL_TRIANGLE_FAN)
-                        bgl.glColor4f(color[0], color[1], color[2], color[3])
-                        for uv in poly:
-                            x, y = context.region.view2d.view_to_region(
-                                uv.x, uv.y, clip=False)
-                            bgl.glVertex2f(x, y)
-                        bgl.glEnd()
+                        coords = [context.region.view2d.view_to_region(uv.x, uv.y, clip=False) for uv in poly]
+                        batch = batch_for_shader(shader, 'TRI_FAN', {"pos": coords})
+                        batch.draw(shader)
+
                 elif sc.muv_uv_inspection_show_mode == 'FACE':
-                    bgl.glBegin(bgl.GL_TRIANGLE_FAN)
-                    bgl.glColor4f(color[0], color[1], color[2], color[3])
-                    for uv in info["subject_uvs"]:
-                        x, y = context.region.view2d.view_to_region(
-                            uv.x, uv.y, clip=False)
-                        bgl.glVertex2f(x, y)
-                    bgl.glEnd()
+                    coords = [
+                        context.region.view2d.view_to_region(
+                            uv.x, uv.y, clip=False) for uv in info["subject_uvs"]]
+                    batch = batch_for_shader(shader, 'TRI_FAN', {"pos": coords})
+                    batch.draw(shader)
 
         # render flipped UV
         if sc.muv_uv_inspection_show_flipped:
-            color = prefs.uv_inspection_flipped_color
+            shader.uniform_float("color", prefs.uv_inspection_flipped_color)
+
             for info in props.flipped_info:
                 if sc.muv_uv_inspection_show_mode == 'PART':
                     for poly in info["polygons"]:
-                        bgl.glBegin(bgl.GL_TRIANGLE_FAN)
-                        bgl.glColor4f(color[0], color[1], color[2], color[3])
-                        for uv in poly:
-                            x, y = context.region.view2d.view_to_region(
-                                uv.x, uv.y, clip=False)
-                            bgl.glVertex2f(x, y)
-                        bgl.glEnd()
-                elif sc.muv_uv_inspection_show_mode == 'FACE':
-                    bgl.glBegin(bgl.GL_TRIANGLE_FAN)
-                    bgl.glColor4f(color[0], color[1], color[2], color[3])
-                    for uv in info["uvs"]:
-                        x, y = context.region.view2d.view_to_region(
-                            uv.x, uv.y, clip=False)
-                        bgl.glVertex2f(x, y)
-                    bgl.glEnd()
+                        coords = [context.region.view2d.view_to_region(uv.x, uv.y, clip=False) for uv in poly]
+                        batch = batch_for_shader(shader, 'TRI_FAN', {"pos": coords})
+                        batch.draw(shader)
 
-        bgl.glDisable(bgl.GL_BLEND)
+                elif sc.muv_uv_inspection_show_mode == 'FACE':
+                    coords = [context.region.view2d.view_to_region(uv.x, uv.y, clip=False) for uv in info["uvs"]]
+                    batch = batch_for_shader(shader, 'TRI_FAN', {"pos": coords})
+                    batch.draw(shader)
+
+        gpu.state.blend_set('NONE')
 
     def invoke(self, context, _):
         if not MUV_OT_UVInspection_Render.is_running(context):
@@ -375,7 +447,8 @@ class MUV_OT_UVInspection_PaintUVIsland(bpy.types.Operator):
                 "MagicUV_PaintUVMaterial_{}".format(i))
             if compat.check_version(2, 80, 0) >= 0:
                 target_mtrl.use_nodes = True
-                output_node = target_mtrl.node_tree.nodes["Material Output"]
+                output_node = next(n for n in target_mtrl.node_tree.nodes
+                                   if n.type == "OUTPUT_MATERIAL")
                 nodes_to_remove = [n for n in target_mtrl.node_tree.nodes
                                    if n != output_node]
                 for n in nodes_to_remove:
